/**
 * Tuxtalador2 -- Tuxtalador rewrite that helps with a system installation.
 * Copyright (C) 2008 Tuxum Secure Systems.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <QFile>
#include <QTextStream>

#include "basemodule.h"
#include "modulemanager.h"

using namespace Tuxtalador2;

ModuleManager::ModuleManager(QObject *parent):QObject(parent){
	step=0;
	nextStep=100;
	
	// Some 'stupid' default values.
	setConfig("LANGUAGE","es");
	setConfig("LOCALE","es_ES");
	setConfig("HOSTNAME","noname");
	setConfig("OSNAME","TuxumOS");
	setConfig("OSRELEASE","2009a1");
	setConfig("OSVERSION","intrepid");
	setConfig("OSDESCRIPTION","TuxumOS 2009a1");
	setConfig("TIMEZONE","Etc/UTC");

	// Parse /etc/installer.config for default env vars to use on the installer.
	QFile f("/etc/installer.config");
	f.open(QFile::ReadOnly);
	QTextStream ts(&f);
	QString l=ts.readLine();
	QString v;
	while (!ts.atEnd()){
		if (!l.startsWith("#") && l.contains("=")==1){
			v=l.section("=",1,1);
			if (v.startsWith("\""))
				v=v.mid(1,v.length()-2);
			setConfig(l.section("=",0,0),v);
		}
		l=ts.readLine();
	}
}

/**
 * @short Adds a wizard at a given step
 */
bool ModuleManager::addWizard(BaseModule *module, int where){
	ModulePosition newmod(module,where);
	
	int atpos=0;
	foreach(ModulePosition pos, wizards){
		if (where<pos.position)
			break;
		atpos++;
	}
	//qDebug("%s:%d insert %d at %d",__FILE__,__LINE__,where,atpos);
	wizards.insert(atpos,newmod);

	return true;
}

/**
 * @short Sends the advice of current step finished, and starts the next.
 */
void ModuleManager::endStepSlot(){
	emit endStep(step);
}

/**
 * @short Go to previous wizard
 */
void ModuleManager::goPrevious(){
	ModulePosition np(NULL,0);
	nextStep=step;
	foreach(ModulePosition p, wizards){
		if (p.position<step)
			np=p;
		else{
			BaseModule *nw=np.module;
			if (nw){
				step=np.position;
				//qDebug("%s:%d Selected widget %d",__FILE__,__LINE__,np.position);
				emit setWizard(nw,step);
			}
			return;
		}
	}
}

/**
 * @short Goes to the next wizard
 */ 
void ModuleManager::goNext(){
	bool okCurrent=false;
	nextStep=100;
	foreach(ModulePosition p, wizards){
		if (okCurrent){ // on the next to the matching one, I set next and return.
			nextStep=p.position;
			return;
		}
		if (p.position>step){
			BaseModule *nw=p.module;
			if (nw){
				step=p.position;
				qDebug("%s:%d Selected widget %d",__FILE__,__LINE__,p.position);
				emit setWizard(nw,step);
				okCurrent=true;
			}
		}
	}
}

/**
 * @short Goes to the given position, if it exists. Else it returns false.
 */
bool ModuleManager::go(int newPos){
	bool okCurrent=false;
	nextStep=100;
	foreach(ModulePosition p, wizards){
		if (okCurrent){ // on the next to the matching one, I set next and return.
			nextStep=p.position;
			return true;
		}
		if (p.position==newPos){
			BaseModule *nw=p.module;
			if (nw){
				step=newPos;
				emit setWizard(nw,step);
				okCurrent=true;
			}
		}
	}
	return false;
}


/**
 * @short Gets a value from the global config.
 */
QString ModuleManager::getConfig(const QString &key){
	return globalConfig[key];
}
/**
 * @short Sets a value to the global config.
 */
void ModuleManager::setConfig(const QString &key, const QString &value){
	qDebug("%s:%d set config %s to %s",__FILE__,__LINE__,
			(char*)key.toUtf8().data(),(char*)value.toUtf8().data());
	globalConfig[key]=value;
}

/**
 * @short Returns all the keys on the config dict.
 */
QStringList ModuleManager::getConfigKeys(){
	return globalConfig.keys();
}


/**
 * @short Checks if I am currently at the last step of the installation, to allow clean exit
 *
 * It runs over all the steps (that is ordered), and if at  current sets the atLast bool to true,
 * If there is no more steps, it keeps that value, else it reverts to false.
 */
bool ModuleManager::atLastStep(){
	bool atLast=false;
	foreach(ModulePosition p, wizards){
		atLast=(p.position==step);
	}
	return atLast;
}

#define MIN(a,b) ((a<b) ? (a) : (b))

/**
 * @short Given a partial progress (0-100) updates the bar to reflect that progress.
 */
void ModuleManager::partialProgress(int percent){
	int currentProgress=(((nextStep-step)*percent)/100)+step;
	emit progress(MIN(currentProgress,nextStep));
}
