/*
TRANSLATOR Tuxtalador2::Install
*/
/**
 * Tuxtalador2 -- Tuxtalador rewrite that helps with a system installation.
 * Copyright (C) 2008 Tuxum Secure Systems.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <sys/stat.h>
#include <sys/types.h>

#include <QFile>
#include <QLabel>
#include <QTextStream>
#include <QMessageBox>
#include <QApplication>

#include "install.h"
#include "../process.h"


using namespace Tuxtalador2;

Install::Install(ModuleManager *mm): BaseModule(mm){
	qDebug("%s:%d Loaded install",__FILE__,__LINE__);

	mm->addWizard(this, 50);
	mm->addWizard(this, 51);

	connect(this,SIGNAL(log(const QString&,QColor)), mm, SIGNAL(log(const QString &,QColor)));
	packagesInstalled=0;
	totalPackages=0;
}

/**
 * @short Reads the packages to install, and installs them.
 */
QWidget *Install::wizard(int step){
	if (step==50){
		QFile f("/target/etc/apt/sources.list");
		f.open(QFile::WriteOnly);
		QString mirror;
		if (getenv("DEB_MIRROR"))
			mirror=getenv("DEB_MIRROR");
		else
			mirror="file:///cdrom/tuxum";

		QString dist;
		if (getenv("DIST"))
			dist=getenv("DIST");
		else
			dist="intrepid";

		f.write(QString("deb %1 %2 main\n").arg(mirror).arg(dist).toLatin1());
		mkdir ("/target/proc",0777);
		mkdir ("/target/cdrom",0777);
		mkdir ("/target/sys",0777);
		system("chroot /target mount -t proc proc proc");
		system("chroot /target mount -t sysfs sysfs sys");
		system("chroot /target ifconfig lo 127.0.0.1 up");
		system("mount /dev/cdrom /target/cdrom");
		system("mv /target/sbin/initctl /target/sbin/initctl.upstart");
		system("touch /target/sbin/initctl");
		chmod("/target/sbin/initctl",0755);
		
		manager->goNext();
	}
	else if (step==51){
		packages.clear();
	
		QFile pack("/lib/packages");
		pack.open(QFile::ReadOnly);
		QTextStream packStream(&pack);
		while (!packStream.atEnd()){
			QString name=packStream.readLine().trimmed();
			if (!name.isEmpty() && !name.startsWith("#")){
				qDebug("%s:%d %s",__FILE__,__LINE__,(char*)name.toUtf8().data());
				packages.append(name);
			}
		}
		packagesLeft=packages;
		totalPackages=packagesLeft.count();

		emit log(QString("%1 packages to install").arg(packages.count()),QColor("blue"));
	
		currentPackageLabel=new QLabel(NULL);
		currentPackageLabel->setWordWrap(true);
		currentPackageLabel->setAlignment(Qt::AlignTop);
	

		emit log(QString("Update apt-get database"),QColor("green"));
		process=new Process(
						QString("chroot /target apt-get update"),
						this,false,false);
		connect(process,SIGNAL(finished(int)),this,SLOT(installFinished(int)));
		currentPackageLabel->setText(tr("Updating..."));

		//installPackage();
	
		return currentPackageLabel;
	}
	return NULL;
}

/// The title page
QString Install::label(int step){
	if (step==50)
		return tr("Prepare APT");
	return tr("Install applications");
}


/**
 * Creates aprocess to install currentIndex package
 */
void Install::installPackage(){
	if (packagesLeft.count()==0){
		emit log(tr("No more packages to install"),QColor("green"));
		manager->goNext();
		return;
	}

	qDebug("%s:%d %d packages to install",__FILE__,__LINE__,packagesLeft.count());

	if (process){
		qDebug("%s:%d remove outdated Process",__FILE__,__LINE__);
		process->deleteLater();
	}
	emit log(tr("Starting install process"),QColor("green"));
	setenv("DEBIAN_FRONTEND","noninteractive",1);
	process=new Process(
			QString("chroot /target apt-get install  -y --force-yes -o Dpkg::Options=--force-all %1").
					arg(packagesLeft.join(" ")),
			this,false,false);
	connect(process,SIGNAL(finished(int)),this,SLOT(installFinished(int)));
	connect(process,SIGNAL(log(const QString &, QColor)), 
			this, SLOT(readStdout(const QString &, QColor)));

	//currentPackageLabel->setText(packages[packageIndex]);
}

/**
 * @short apt-get finished, depending on the error code it continues, or retries.
 */
void Install::installFinished(int code){
	emit log(tr("Install process finished"),QColor("green"));
	qDebug("%s:%d finished %d",__FILE__,__LINE__,code);
	if (code){
		emit log(tr("Error %d").arg(code),QColor("red"));
		int res=QMessageBox::question(NULL,tr("The process finished abnormally"),
				tr("Current step did not finish corectly. Do you want to retry it,\n",
				   "or just got o the next step?"),QMessageBox::Retry,QMessageBox::Ignore);
		if (res==QMessageBox::Retry){
			installPackage();
		}
		else{
			//qApp->exit(1);
			manager->goNext();
		}
	}
	else if (packagesLeft.count()>0){
		emit log(tr("Still %1 packages to install").arg(packagesLeft.count()),QColor("green"));
		emit log(tr("packages left: %1").arg(packagesLeft.join(" ")),QColor("purple"));

#if 0
		int res=QMessageBox::question(NULL,tr("The process finished abnormally"),
				tr("Current step did not finish corectly. Do you want to retry it,\n",
				   "or just got to the next step?\n%1 packages left").arg(packagesLeft.count()),
				QMessageBox::Retry,QMessageBox::Ignore);
		if (res==QMessageBox::Retry){
			installPackage();
		}
		else{
			//qApp->exit(1);
			manager->goNext();
		}
#else
		installPackage();
#endif
	}
	else{
		emit log(tr("Going to next section"),QColor("green"));
		manager->goNext();
	}
}

/**
 * @short Updates progress bar.
 */
void Install::readStdout(const QString &strs, QColor){
	foreach(QString str, strs.split("\n")){
		if (str.startsWith("Unpacking") || str.startsWith("Setting") ||
			str.contains("already") ){
			packagesInstalled++;
			currentPackageLabel->setText(str);
			QString packageName;
			if (str.startsWith("Unpacking"))
				packageName=str.section(' ',1,1);
			else if (str.startsWith("Setting"))
				packageName=str.section(' ',2,2);
			else
				packageName=str.section(' ',0,0);
			/*
			if (totalPackages && packagesLeft.contains(packageName)){
				// Uses alphabetical progress... it is more or less correct..
				int p=0;
				if (str.startsWith("Setting"))
					p=50;
				p+=(((char)packageName[0].toAscii())-'a')*50/27;
				p+=(((char)packageName[1].toAscii())-'a')*50/(27*27);
				
			}
			*/
			qDebug("%s:%d installed %s",__FILE__,__LINE__,(char*)packageName.toLatin1().data());
			//int precount=packagesLeft.count();
			packagesLeft.removeAll(packageName);
			
			manager->partialProgress((totalPackages-packagesLeft.count())*50/totalPackages);
			emit log(tr("Progress %1 * 100 / %2").arg((totalPackages-packagesLeft.count())/2).arg(totalPackages), QColor("blue"));
		}
		else if (str.startsWith("Get")){
			//totalPackages++;
		}
	}
}
